#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#*                                                                           *
#*    This file is part of the test engine for the 11th DIMACS challenge     *
#*                                                                           *
#* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

SHELL=$1
BINNAME=$2
TSTNAME=$3
TIMELIMIT=$4
THREADS=$5
QUEUE=$6
PPN=$7
EXCLUSIVE=$8
CONTINUE=$9

# construct paths
SCRIPTPATH=`pwd`
BINPATH=$SCRIPTPATH/bin
RESULTSPATH=$SCRIPTPATH/results
SOLUTIONPATH=$SCRIPTPATH/results/solutions
TSTPATH=$SCRIPTPATH/testsets

# check if the solver link (binary) exists
if test ! -e $BINPATH/$BINNAME
then
    echo "ERROR: solver link <$BINNAME> does not exist in <bin> folder; see bin/README"
    exit;
fi

# check if the test set file/link exists
if test ! -e $TSTPATH/$TSTNAME.test
then
    echo "ERROR: test set file/link <$TSTPATH/$TSTNAME.test> does not exist"
    exit;
fi


# new environment variables defined by this script:
NICE=""
ACCOUNT="mip"
CLUSTERQUEUE=$QUEUE

# check if queue has been defined
if test "$QUEUE" = ""
then
    echo Skipping test since the queue name has not been defined.
    exit
fi

# check if number of nodes has been defined
if test "$PPN" = ""
then
    echo Skipping test since the number of nodes has not been defined.
    exit
fi

#define clusterqueue, which might not be the QUEUE, because this might be an alias for a bunch of QUEUEs
if test $CLUSTERQUEUE = "dbg"
then
    CLUSTERQUEUE="mip-dbg,telecom-dbg"
    ACCOUNT="mip-dbg"
elif test $CLUSTERQUEUE = "telecom-dbg"
then
    ACCOUNT="mip-dbg"
elif test $CLUSTERQUEUE = "mip-dbg"
then
    ACCOUNT="mip-dbg"
elif test $CLUSTERQUEUE = "opt-low"
then
    CLUSTERQUEUE="opt"
    NICE="--nice=10000"
elif test $CLUSTERQUEUE = "M620x"
then
    CLUSTERQUEUE="M620,M620v2,M620v3"
fi

# check if the slurm blades should be used exclusively
if test "$EXCLUSIVE" = "true"
then
    EXCLUSIVE=" --exclusive"
    if test $CLUSTERQUEUE = "opt"
    then
        CLUSTERQUEUE="M610"
    fi
else
    EXCLUSIVE=""
fi

# we add 100% to the hard time limit and additional 600 seconds in case of small time limits
HARDTIMELIMIT=`expr \`expr $TIMELIMIT + 600\` + $TIMELIMIT`

#format is (d-)HH:MM:SS
TMP=`expr $HARDTIMELIMIT`
HARDTIMELIMIT=""
DIVISORS=(60 60 24)
for((i=0; i<=2; i++))
do
    printf -v HARDTIMELIMIT "%02d${HARDTIMELIMIT}" `expr ${TMP} % ${DIVISORS[i]}`
    # separate the numbers by colons except for the last (HH hours)
    if test $i -lt 2
    then
        HARDTIMELIMIT=":${HARDTIMELIMIT}"
    fi
    TMP=`expr ${TMP} / ${DIVISORS[i]}`
done
if test ${TMP} -gt 0
then
    HARDTIMELIMIT=${TMP}-${HARDTIMELIMIT}
fi

# check if the results folder exists, if not create it
if test ! -e $RESULTSPATH
then
    mkdir $RESULTSPATH
fi

# check if the logs folder exists, if not create it
if test ! -e $RESULTSPATH/logs
then
    mkdir $RESULTSPATH/logs
fi

# check if the solution folder exists, if not create it
if test ! -e $SOLUTIONPATH
then
    mkdir $SOLUTIONPATH
fi

# construct name of output and results file
BASENAME=$RESULTSPATH/$TSTNAME.$BINNAME.$THREADS.$TIMELIMIT.$QUEUE
EVALFILE=$BASENAME.eval
OUTFILE=$BASENAME.out
RESFILE=$BASENAME.res

echo > $EVALFILE

# post system information and current time into the output file
#uname -a > $OUTFILE
#date >> $OUTFILE

# loop over all instance names which are listed in the test set file name
for i in `cat $TSTPATH/$TSTNAME.test`
do
    # check if the current instance exists
    if test -f $i
    then
	FILENAME=`basename $i .gz`
	FILENAME=`basename $FILENAME .stp`

	SOLFILE=$SOLUTIONPATH/$TSTNAME.$FILENAME.$BINNAME.$THREADS.$TIMELIMIT.$QUEUE.sol

	echo $SOLFILE >> $EVALFILE

	if test "true" = "$CONTINUE"
	then
	    if test -f $SOLFILE
	    then
		echo "skipping instance $FILENAME: solution file exists"
		continue
	    fi
	fi

        export CLIENTTMPDIR
        export BINPATH
	export BINNAME
	export INSTANCE=$i
	export TIMELIMIT
	export THREADS
	export TSTNAME
	export FILENAME
	export SCRIPTPATH
	export OUTFILE
	export SOLFILE
	export SHELL
	export QUEUE
	export CLIENTTMPDIR=/usr/local/tmp
	export USER=$USER

	sbatch --job-name=${TSTNAME}$BINNAME -p $CLUSTERQUEUE -A $ACCOUNT $NICE --time=${HARDTIMELIMIT} ${EXCLUSIVE} --output=/dev/null $SCRIPTPATH/scripts/runjob.sh
    fi
done #2>&1 | tee -a $OUTFILE

#date >> $OUTFILE

#$SCRIPTPATH/scripts/parse.sh $OUTFILE | tee $RESFILE
